SystemOrganization addCategory: #BBCRadioRecorder!

Object subclass: #BBCEpisode
	instanceVariableNames: 'brandParent brandElement episodeElement seriesElement brandPid brandTitle brandPosition brandChildCount seriesParent seriesPid seriesTitle seriesPosition seriesChildCount episodePid episodeTitle episodePosition episodeSynopsisShort episodeSynopsisMedium episodeSynopsisLong episodeVersions duration canonicalPid ownership genre'
	classVariableNames: 'DefaultDestinationFolder DefaultFfmpegPath DefaultRtmpdumPath'
	poolDictionaries: ''
	category: 'BBCRadioRecorder'!

!BBCEpisode commentStamp: 'jrm 5/22/2016 14:46' prior: 0!
#BBCEpisode is a class that manages the information about a program required to listen to, or record, a BBCRadio program episode. It is an object used to provide the information required to build a command to be used to record a BBC radio stream. A BBCEpisode pid can be obtained from the XML contents for an availability, schedule or genra; Those sources have different XML footprints and are used to provide data for a program selection list from which a program can be selected and an episode pid can be obtained. 

Instance Variables

canonicalPid - required to determine mediaSelector url
brandElement - anXML
episodeElement 
seriesElement 
episodePid 
episodeDuration 
episodePosition 
episodeSynopsisShort 
episodeTitle 
brandChildCount 
brandPid 
brandPosition 
brandTitle 
seriesChildCount 
seriesPosition 
seriesTitle 
seriesPid
!

!BBCEpisode class methodsFor: 'instance creation' stamp: 'jrm 5/22/2016 14:59'!
newFor: anEpisodePid

	^ self new newFor: anEpisodePid.
! !

!BBCEpisode class methodsFor: 'instance creation' stamp: 'jrm 5/10/2017 17:54'!
newForUrl: aUrlString
	"Convenience method for cut and paste episode pids"
	"self newForUrl: 'http://www.bbc.co.uk/programmes/b007lzzd'"
		| anEpisodePid |
	
	anEpisodePid := aUrlString subStrings:'/'.
	^ self new newFor: (anEpisodePid last).
! !

!BBCEpisode methodsFor: 'accessing' stamp: 'jrm 7/31/2016 14:18'!
canonicalPid

	^canonicalPid! !

!BBCEpisode methodsFor: 'accessing' stamp: 'jrm 8/12/2016 17:36'!
composeFilename

	"build a reasonable string to be used as a filename in a command "
	| fileName option tempFileName |
	fileName := self rawFilename.
	tempFileName := ByteString new.
	fileName := fileName select: [ :each | each value ~= Character space].
	fileName := fileName select: [ :each | each value ~= (Character characterTable at: 64)].
	fileName do: [:each | each = $/ ifTrue: [each := '-'].
						   each = $: ifTrue: [each := '-'].
					         each = $' ifTrue: [each := '-' ]].
	"tempFileName := SequenceableCollection streamContents: [fileName]."
	fileName  replaceAll:  $/  with: $-. 
	fileName replaceAll:  $:  with: $-.
	fileName replaceAll:  $'  with: $-.
	fileName replaceAll:  $, with: $-.
	"Ask for the desired option"
	option := UserDialogBoxMorph confirm: 'File Name for Stream' withCRs title: 'Create new, or look up existing?' trueChoice: 'Lookup' falseChoice: 'Create'.  
	option ifTrue: [ fileName := self fileNameFromString: (self fileNameFromDirectory: fileName )] 
			ifFalse:[fileName := self fileNameFromString: fileName].	
	^ fileName   " move the suffix attachement to another place     , '.mp3'"! !

!BBCEpisode methodsFor: 'accessing' stamp: 'jrm 8/14/2016 20:10'!
episodePid
	^episodePid! !

!BBCEpisode methodsFor: 'accessing' stamp: 'jrm 8/14/2016 18:48'!
episodePosition

	^ episodePosition! !

!BBCEpisode methodsFor: 'accessing' stamp: 'jrm 8/14/2016 18:47'!
episode_id
	^ episodePid! !

!BBCEpisode methodsFor: 'accessing' stamp: 'jrm 8/12/2016 17:37'!
fileNameFromDirectory: aString
	" Let me select a name from a list so I can keep the file names for a list similar. I will edit the file 	name that I select in a later step"
	"BBCRecorderApp fileNameFromList"
	
	| fileList fileNameLIst fileName|
	fileList := (FileList newOn: (FileDirectory on: '/Volumes/MediaVault/BBC')) entriesMatching: '*.mp3'.
	fileNameLIst := fileList collect: [ :each | each name].
	fileName := ListChooser chooseItemFrom: fileNameLIst title: 'Hint: ', aString.
	"Strip file type indicator string .mp3 from end of file"
	^ fileName == nil ifTrue: [^fileName] ifFalse: [^ fileName copyFrom: 1 to: (fileName size -4)]
! !

!BBCEpisode methodsFor: 'accessing' stamp: 'jrm 8/14/2016 19:38'!
fileNameFromString: aString
	"Modify the title string to create a useful file name to store the transcribed stream."
	
	
	^ FillInTheBlankMorph  request:  '.Mxxx will be appended to file name' initialAnswer: aString centerAt: 500@500.! !

!BBCEpisode methodsFor: 'accessing' stamp: 'jrm 8/14/2016 18:20'!
genre
	^genre! !

!BBCEpisode methodsFor: 'private' stamp: 'jrm 6/25/2017 20:54'!
newFor: anEpisodePid

	| episodeDocument episodeUrl episodeStream |
	
	episodeUrl := 'http://www.bbc.co.uk/programmes/', anEpisodePid, '.json'.
	episodeStream :=HTTPClient httpGet: episodeUrl.
	episodeStream  isString ifTrue: [^ (self error: ' in get from', episodeUrl)].
	
	episodeDocument  := Json readFrom: (episodeStream).
	episodeElement := episodeDocument at: 'programme'.

	episodePid := episodeElement at: 'pid'.
	
	episodePosition :=(episodeElement at: 'position').
	episodePosition isInteger ifTrue:[episodePosition :=episodePosition asString] ifFalse: [episodePosition := ''].
	
	episodeTitle := episodeElement at: 'title'.
	episodeSynopsisShort := episodeElement at: 'short_synopsis'.
	episodeSynopsisMedium := episodeElement at: 'medium_synopsis'.
	episodeSynopsisLong := episodeElement at: 'long_synopsis'.
	episodeVersions :=  (episodeElement at: 'versions') at:1.
	duration := (episodeVersions at: 'duration').
	canonicalPid := (episodeVersions at: 'pid') .
	episodeElement at: 'ownership' ifPresent:[:value |value := ((episodeElement at: 'ownership') at: 'service') at: 'title'] ifAbsent: [''].

	((episodeElement at: 'categories')size ~= 0) ifTrue: [genre := ((episodeElement at: 'categories') at: 1) at: 'title'] ifFalse: [genre := 'unknown' ].
	
	seriesParent := (episodeElement at: 'parent').
	seriesParent = nil ifFalse: [
	seriesElement := seriesParent at: 'programme'.
	seriesElement = nil ifFalse: [
	seriesPid := seriesElement at: 'pid'.
	seriesPosition :=(seriesElement at: 'position').
	seriesPosition isInteger ifTrue:[seriesPosition :=seriesPosition asString] ifFalse: [seriesPosition := ''].
	seriesTitle := seriesElement at: 'title'.
	seriesChildCount := seriesElement at: 'expected_child_count'.
	seriesChildCount isInteger ifTrue:[seriesChildCount :=seriesChildCount asString]].

	brandParent := (seriesElement at: 'parent').
	brandParent = nil ifFalse: [
	brandElement := brandParent at: 'programme'.
	brandElement = nil ifFalse: [
	brandPid := brandElement at: 'pid'.
	brandPosition :=(brandElement at: 'position').
	brandPosition isInteger ifTrue:[brandPosition :=brandPosition asString] ifFalse: [brandPosition := ''].
	brandTitle := brandElement at: 'title'.
	brandChildCount := brandElement at: 'expected_child_count'.
	brandChildCount isInteger ifTrue:[brandChildCount :=brandChildCount asString] ifFalse:[brandChildCount :='']]]].
	
	" Make sure nils are set to blank strings to avoid problems for consumers of the data "
	seriesTitle == nil ifTrue: [seriesTitle := ''].
	seriesPosition == nil ifTrue: [seriesPosition := ''].
	seriesChildCount == nil ifTrue: [seriesChildCount := ''].
	brandTitle == nil ifTrue: [brandTitle := ''].
	brandPosition == nil ifTrue: [brandPosition := ''].
	brandChildCount == nil ifTrue: [brandChildCount := ''].


^ self ! !

!BBCEpisode methodsFor: 'accessing' stamp: 'jrm 8/14/2016 17:10'!
ownership
	^ownership! !

!BBCEpisode methodsFor: 'accessing' stamp: 'jrm 8/12/2016 17:34'!
rawFilename
" Build a space delimited string from the raw title data"
	| displayNofM displayBrandTitle displaySeriesTitle displayEpisodeTitle episodeDashCheck dotSeriesChildCount |

	seriesChildCount ~= '' ifTrue: [dotSeriesChildCount := '.',seriesChildCount] ifFalse: [dotSeriesChildCount :=''].
	episodePosition = '' ifTrue: [ displayNofM := ''] 
						ifFalse: [ displayNofM := '.', episodePosition, dotSeriesChildCount].
	seriesPosition = '' ifFalse: [displayNofM := '.', seriesPosition, '.', episodePosition, '.', seriesChildCount].
	( 'Episode*' match: episodeTitle ) ifTrue: [displayEpisodeTitle := ''] ifFalse: [displayEpisodeTitle := episodeTitle].
	('Series*'  match: seriesTitle) ifTrue: [displaySeriesTitle := ''] ifFalse: [displaySeriesTitle := seriesTitle].
	brandTitle = '' ifFalse: [displayBrandTitle := brandTitle] ifTrue: [displayBrandTitle := ''].
	episodeDashCheck := displayBrandTitle,displaySeriesTitle, displayNofM.
	(episodeDashCheck ~= '' ) & (displayEpisodeTitle ~= '')  ifTrue: [displayEpisodeTitle := '-', displayEpisodeTitle].
	^ displayBrandTitle,displaySeriesTitle, displayNofM, displayEpisodeTitle
	! !

!BBCEpisode methodsFor: 'accessing' stamp: 'jrm 8/14/2016 17:13'!
synopsis

	^ episodeSynopsisShort! !

!BBCEpisode methodsFor: 'accessing' stamp: 'jrm 8/14/2016 19:49'!
track
	^episodePosition, '/', seriesChildCount! !

Object subclass: #BBCEpisodeMetadata
	instanceVariableNames: 'title author album_artist album grouping composer year track comment genre copyright description synopsis show episode_id network lyrics'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'BBCRadioRecorder'!

!BBCEpisodeMetadata class methodsFor: 'as yet unclassified' stamp: 'jrm 8/14/2016 16:55'!
newFor: episode
	"create a string containing the metadata to be used when creating a command line string in BBCEpisodeRecorder. The string returned from this method will be included in the command line string created by BBCEpisodeRecorder"

 	self new newFor: episode.! !

!BBCEpisodeMetadata methodsFor: 'as yet unclassified' stamp: 'jrm 8/14/2016 18:30'!
formatKey: keyString value: valueString
	"return the value expected in the FFmpeg command line"

	^ ' -metadata ', keyString, '="', valueString, '"'.! !

!BBCEpisodeMetadata methodsFor: 'as yet unclassified' stamp: 'jrm 8/14/2016 20:25'!
newFor: episode

	"create a string containing the metadata to be used when creating a command line string in BBCEpisodeRecorder. The string returned from this method will be included in the command line string created by BBCEpisodeRecorder"

	| metadataString |

	genre := self formatKey: 'genre' value: episode genre.
	network := self formatKey: 'network' value: episode ownership.
	description := self formatKey: 'description' value: episode synopsis.
	synopsis := self formatKey: 'synopsis' value: episode synopsis.
	episode_id := self formatKey: 'episode_id' value: episode episode_id.
	track := self formatKey: 'track' value: episode track.
	
	"title := self formatKey: 'title' value: (episode fileNameFromString: (episode rawFilename))."
	metadataString := genre, network, description, synopsis, episode_id, track.
	^ metadataString! !

Object subclass: #BBCEpisodeRecorder
	instanceVariableNames: 'mediaDocument mediaAac connection supplier protocol authExpires authString priority server application identifier flvFileName cmdLine eosp'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'BBCRadioRecorder'!

!BBCEpisodeRecorder commentStamp: 'jrm 4/30/2016 13:29' prior: 0!
A BBCEpisodeRecorder is builder of the command line which is required to record an episode of a BBC radio program. 
This version requires RTMPDUMP and FFMPEG programs to be installed. This program was built and tested on a Mac.

I am trying to design it as a free standing program that can be used alone or in conjunction with the BBCRecorderApp.

The url for the MediaSelector is a moving target.

!

!BBCEpisodeRecorder class methodsFor: 'sandbox' stamp: 'jrm 1/3/2017 21:17'!
recTestCmd
	"self recTestCmd"
|cmdLine in out err proc |

	cmdLine := '
/opt/local/bin/rtmpdump -r "rtmp://cp107663.edgefcs.net:1935/ondemand?auth=daEdJcnbvbgdnbRdyalbncicHcKdudGd9cp-byBhxE-bWG-GpoEBoCqICqHrwD&aifp=v001&slist=/prod_af_mp4_heaacv1_48/iplayerstream/l2o/b00fnxbt_98e79f9e-f5bd-4386-abc1-2f6f5f5779a2.mp4" -a "ondemand?auth=daEdJcnbvbgdnbRdyalbncicHcKdudGd9cp-byBhxE-bWG-GpoEBoCqICqHrwD&aifp=v001&slist=/prod_af_mp4_heaacv1_48/iplayerstream/l2o/b00fnxbt_98e79f9e-f5bd-4386-abc1-2f6f5f5779a2.mp4" -y "mp4:/prod_af_mp4_heaacv1_48/iplayerstream/l2o/b00fnxbt_98e79f9e-f5bd-4386-abc1-2f6f5f5779a2.mp4" -o /Volumes/MediaVault/BBC/82141057.flv ; /opt/local/bin/ffmpeg -loglevel quiet  -i /Volumes/MediaVault/BBC/82141057.flv -metadata genre="Relationships & Romance" -metadata network="BBC Radio 4" -metadata description="Middle-class estate agent Edith meets Charles, the extremely eligible earl." -metadata synopsis="Middle-class estate agent Edith meets Charles, the extremely eligible earl." -metadata episode_id="b00fnxcb" -metadata track="1/5" -c copy -copyts /Volumes/MediaVault/BBC/"testjunkx".mp4 ; rm /Volumes/MediaVault/BBC/82141057.flv'.
	
	

	in := FileStream  readOnlyFileNamed: '/etc/hosts'.
	out := FileStream newFileNamed: '/Volumes/MediaVault/BBC/deleteMe.out'.
	err := FileStream newFileNamed: '/Volumes/MediaVault/BBC/deleteMe.err'.
	proc := UnixProcess
				forkJob: ExternalUnixOSProcess bashShellPath
				arguments: (Array with: '-c' with: cmdLine)
				environment: nil
				descriptors: (Array
						with: in
						with: out
						with: err).
	in close.
	out close.
	err close.
	proc ifNil: [self noAccessorAvailable].
	
	^ proc! !

!BBCEpisodeRecorder class methodsFor: 'sandbox' stamp: 'jrm 1/3/2017 20:29'!
recTestCmdStreams
"self recTestCmdStreams"

|cmdLine in out err proc |

	cmdLine := '/opt/local/bin/rtmpdump -r "rtmp://cp107663.edgefcs.net:1935/ondemand?auth=daEcObtdpaCcddhcea3coa5aJdNcfczaqc4-byxSvB-bWG-BouGFnEpIEuGrwH&aifp=v001&slist=/prod_af_mp4_heaacv1_48/iplayerstream/p04k2pn2_b08558k0_1482397854075.mp4" -a "ondemand?auth=daEcObtdpaCcddhcea3coa5aJdNcfczaqc4-byxSvB-bWG-BouGFnEpIEuGrwH&aifp=v001&slist=/prod_af_mp4_heaacv1_48/iplayerstream/p04k2pn2_b08558k0_1482397854075.mp4" -y "mp4:/prod_af_mp4_heaacv1_48/iplayerstream/p04k2pn2_b08558k0_1482397854075.mp4" -o /Volumes/MediaVault/BBC/472596.flv ; /opt/local/bin/ffmpeg -loglevel quiet  -i /Volumes/MediaVault/BBC/472596.flv -metadata genre="Bulletins" -metadata network="BBC Radio 4" -metadata description="News from BBC Radio 4." -metadata synopsis="News from BBC Radio 4." -metadata episode_id="b08558ly" -metadata track="/" -c copy -copyts /Volumes/MediaVault/BBC/"testjunk".mp4 ; rm /Volumes/MediaVault/BBC/472596.flv'.
	
	in := WriteStream on: String new.
	out := WriteStream on: String new.
	err := WriteStream on: String new.
	proc := UnixProcess
				forkJob: ExternalUnixOSProcess bashShellPath
				arguments: (Array with: '-c' with: cmdLine)
				environment: nil
				descriptors: (Array
						with: in
						with: out
						with: err).
	in close.
	out close.
	err close.
	proc ifNil: [self noAccessorAvailable].
	
	^ proc! !

!BBCEpisodeRecorder class methodsFor: 'accessing' stamp: 'jrm 6/4/2017 20:49'!
recordEpisode: aBBCEpisode
     "BBCEpisodeRecorder recordEpisode:  (BBCEpisode newFor: 'b0184s3k')"
	"BBCEpisodeRecorder recordEpisode: (BBCEpisode newForUrl: 'http://www.bbc.co.uk/programmes/b08svvb4')"

	self new recordEpisode: aBBCEpisode.

	

! !

!BBCEpisodeRecorder class methodsFor: 'accessing' stamp: 'jrm 4/30/2017 09:19'!
recordEpisodeCanonical: aBBCEpisodeCanonicalPid
     "BBCEpisodeRecorder recordEpisodeCanonical:  (BBCEpisode newFor: 'b0184s3k')"

	self new recordEpisodeCanonical: aBBCEpisodeCanonicalPid.

	

! !

!BBCEpisodeRecorder methodsFor: 'as yet unclassified' stamp: 'jrm 11/8/2017 06:25'!
recordEpisode: aBBCEpisode

	| dblQuote defaultDestination dQuote  mediaStream randomString 
	 mp4Filename filename metadataString mediaStreamCheck |
	"sort out filename creation. filename was passed as a parm in original version"
	dblQuote := '"'.
	mp4Filename :=  dblQuote, aBBCEpisode composeFilename, dblQuote.
	filename := mp4Filename, '.mp4'.
	defaultDestination := BBCRecorderApp defaultDestination.
	randomString := (((Random new next ) * 100000000) asInteger) asString.
	dQuote := '"'. "double quote"
	mediaStream:= HTTPClient httpGet: 'http://open.live.bbc.co.uk/mediaselector/5/select/version/2.0/mediaset/pc/vpid/', aBBCEpisode canonicalPid.
	mediaStreamCheck := mediaStream className.
	 "findString: 'selectionunavailable'."
	mediaStreamCheck = 'ByteString'  ifTrue:[nil error: 'BBC File is missing. Abandon this message to continue. This is a BBC Problem. I can not fix it!!    ' ,  Character cr, Character cr, mediaStream].
	mediaDocument := XMLDOMParser parseDocumentFrom: (mediaStream).
	mediaAac := mediaDocument firstTagNamed: #media with: [:each | (each attributeAt: #bitrate) = '48'].
	mediaAac isNil ifTrue:[mediaAac := mediaDocument firstTagNamed: #media with: [:each | (each attributeAt: #bitrate) = '32']].
	mediaAac isNil ifTrue:[mediaAac := mediaDocument firstTagNamed: #media with: [:each | (each attributeAt: #bitrate) = '128']].
	
	
 	connection := (mediaAac firstTagNamed: #connection) attributes.

	supplier  := connection at: #supplier.
	protocol  := connection at: #protocol.
	authExpires   := connection at: #authExpires.
	authString  := connection at: #authString.
	priority  := connection at: #priority.
	server  := connection at: #server.
	application  := connection at: #application.
	identifier  := connection at: #identifier.
	flvFileName := randomString, '.flv'.
	mp4Filename := filename.
	metadataString := BBCEpisodeMetadata new newFor: aBBCEpisode.
"Transcript show: (Character cr asString), flvFileName."
	cmdLine := '/opt/local/bin/rtmpdump -r ' , dQuote, protocol, '://', server, ':1935/',  application, '?', authString, dQuote, ' -a ', dQuote, application, '?', authString, dQuote, ' -y "', identifier, dQuote,  ' -o ', defaultDestination, flvFileName, ' ; /opt/local/bin/ffmpeg -loglevel quiet  -i ', defaultDestination, flvFileName, metadataString, ' -c copy -copyts ', defaultDestination, mp4Filename, ' ; rm ', defaultDestination, flvFileName.	
		
	eosp := ExternalUnixOSProcess forkAndExec: ExternalUnixOSProcess bashShellPath
		arguments: (Array with: '-c' with: cmdLine)
		environment: nil.
"Transcript show: (Character cr asString), eosp."

Transcript show: (Character cr asString), '-----', (Character cr asString), cmdLine.! !

!BBCEpisodeRecorder methodsFor: 'as yet unclassified' stamp: 'jrm 4/30/2017 09:17'!
recordEpisodeCanonical: aBBCEpisodeCanonicalPid

	| defaultDestination dQuote  mediaStream randomString 
	 mp4Filename filename metadataString mediaStreamCheck |
	"sort out filename creation. filename was passed as a parm in original version"
	dQuote := '"'.
	mp4Filename :=  'tempJunk'.
	filename := mp4Filename, '.mp4'.
	defaultDestination := BBCRecorderApp defaultDestination.
	randomString := (((Random new next ) * 100000000) asInteger) asString.
	mediaStream:= HTTPClient httpGet: 'http://open.live.bbc.co.uk/mediaselector/5/select/version/2.0/mediaset/pc/vpid/',  aBBCEpisodeCanonicalPid.
	mediaStreamCheck := mediaStream className.
	 "findString: 'selectionunavailable'."
	mediaStreamCheck = 'ByteString'  ifTrue:[nil error: 'BBC File is missing. Abandon this message to continue. This is a BBC Problem. I can not fix it!!    ' ,  Character cr, Character cr, mediaStream].
	mediaDocument := XMLDOMParser parseDocumentFrom: (mediaStream).
	mediaAac := mediaDocument firstTagNamed: #media with: [:each | (each attributeAt: #bitrate) = '48'].
 	connection := (mediaAac firstTagNamed: #connection) attributes.

	supplier  := connection at: #supplier.
	protocol  := connection at: #protocol.
	authExpires   := connection at: #authExpires.
	authString  := connection at: #authString.
	priority  := connection at: #priority.
	server  := connection at: #server.
	application  := connection at: #application.
	identifier  := connection at: #identifier.
	flvFileName := randomString, '.flv'.
	mp4Filename := filename.
	"metadataString := BBCEpisodeMetadata new newFor: aBBCEpisode."
"Transcript show: (Character cr asString), flvFileName."
	cmdLine := '/opt/local/bin/rtmpdump -r ' , dQuote, protocol, '://', server, ':1935/',  application, '?', authString, dQuote, ' -a ', dQuote, application, '?', authString, dQuote, ' -y "', identifier, dQuote,  ' -o ', defaultDestination, flvFileName, ' ; /opt/local/bin/ffmpeg -loglevel quiet  -i ', defaultDestination, flvFileName, "metadataString," ' -c copy -copyts ', defaultDestination, mp4Filename, ' ; rm ', defaultDestination, flvFileName.	
		
	eosp := ExternalUnixOSProcess forkAndExec: ExternalUnixOSProcess bashShellPath
		arguments: (Array with: '-c' with: cmdLine)
		environment: nil.
"Transcript show: (Character cr asString), eosp."

Transcript show: (Character cr asString), '-----', (Character cr asString), cmdLine.! !

Object subclass: #BBCOutlet
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'BBCRadioRecorder'!

!BBCOutlet commentStamp: 'jrm 3/1/2014 12:52' prior: 0!
A BBCOutlet is required to manage a variety of address patterns used by different BBC outlets. Radio 1 and Radio 4 seem to be the problem children.

http://www.bbc.co.uk/radio1/programmes/schedules/england/today.xml
http://www.bbc.co.uk/1xtra/programmes/schedules/today.xml
http://www.bbc.co.uk/radio2/programmes/schedules/today.xml
http://www.bbc.co.uk/radio3/programmes/schedules/today.xml
http://www.bbc.co.uk/radio4/programmes/schedules/lw/today.xml
http://www.bbc.co.uk/radio4/programmes/schedules/fm/today.xml
http://www.bbc.co.uk/radio4extra/programmes/schedules/today.xml
http://www.bbc.co.uk/5live/programmes/schedules/today
http://www.bbc.co.uk/5livesportsextra/programmes/schedules.xml
http://www.bbc.co.uk/6music/programmes/schedules.xml
http://www.bbc.co.uk/worldserviceradio/programmes/schedules/today.xml

hmmm... new information

http://www.bbc.co.uk/radio4extra/programmes/schedules/this_week.xml
http://www.bbc.co.uk/radio4extra/programmes/schedules/last_week.xml


Instance Variables
!

Object subclass: #BBCRadioAOD
	instanceVariableNames: 'outlet entries listItemSelected'
	classVariableNames: 'OutletUrlStub'
	poolDictionaries: ''
	category: 'BBCRadioRecorder'!

!BBCRadioAOD commentStamp: 'jrm 5/6/2017 17:13' prior: 0!
BBCRadioAOD is used to create and manage a collection of entries which can be used to build  lists for program selection. AOD availability is currently 6May2017 - in XML format.!

!BBCRadioAOD class methodsFor: 'as yet unclassified' stamp: 'jrm 5/12/2017 17:58'!
newFor: anOutlet


	"BBCRadioAvailabilitySchedule newFor: 'radio4extra' "

	|  newSchedule|
	newSchedule := self new.
	anOutlet = nil ifFalse:[
	^ newSchedule newFrom: 'http://www.bbc.co.uk/radio/aod/availability/', anOutlet, '.xml']


	
! !

!BBCRadioAOD class methodsFor: 'as yet unclassified' stamp: 'jrm 5/9/2017 22:47'!
newFrom: aUrl
	"BBCRadioAOD newFrom: 'http://www.bbc.co.uk/radio/aod/availability/radio4extra.xml'"

	^ self new newFrom: aUrl! !

!BBCRadioAOD methodsFor: 'plugList' stamp: 'jrm 5/10/2017 19:26'!
changeSelected: anObject
	
	self halt.! !

!BBCRadioAOD methodsFor: 'as yet unclassified' stamp: 'jrm 6/27/2017 20:10'!
compressEntries
	
	"Compress entries by removing multiple broadcast entries for a single title (shows
	are broadcast more than once)"
	
	| aDictionary temp anOrderedCollection currentTimeUTC |
	
	
	currentTimeUTC := DateAndTime current asUTC.
	aDictionary := Dictionary new.
	anOrderedCollection := OrderedCollection new.
	
	temp := entries reject: [ :each | each availabilityStartDateAndTime > currentTimeUTC].
	temp := temp collect: [:each | aDictionary at: each title put: each].
	aDictionary valuesDo: [ :each | anOrderedCollection add: each].

	^ entries := anOrderedCollection.
	! !

!BBCRadioAOD methodsFor: 'as yet unclassified' stamp: 'jrm 5/9/2017 23:25'!
entries
	
	^entries! !

!BBCRadioAOD methodsFor: 'plugList' stamp: 'jrm 5/10/2017 18:28'!
list

	| oc |
	
	oc := OrderedCollection new.
	oc do: [:each | oc add: each availabilityStart,  Character space, each canonicalPid, Character space, each episodePid, Character space, each title, Character cr ].
	^ oc
	! !

!BBCRadioAOD methodsFor: 'plugList' stamp: 'jrm 5/10/2017 19:29'!
listItemSelected

	listItemSelected ifNil:[listItemSelected := 1].

	^listItemSelected! !

!BBCRadioAOD methodsFor: 'as yet unclassified' stamp: 'jrm 5/12/2017 18:00'!
newFrom: aUrl
	| xmlStream anXMLDocument root rootElements entryElements |
	
	entries := OrderedCollection new.
	xmlStream := HTTPClient httpGet: aUrl.
	anXMLDocument  := XMLDOMParser parseDocumentFrom: (xmlStream).
	root := anXMLDocument root.
	rootElements := root elements.
	entryElements := rootElements select: [:each | each name = 'entry'].
	entryElements do: [:each | entries add:(BBCRadioAODEntry newFrom: each)].
	self compressEntries.
	 entries sort:[:a :b | a availabilityStart > b availabilityStart].
	
 	^ entries.

	

	

	! !

Object subclass: #BBCRadioAODEntry
	instanceVariableNames: 'canonicalPid key episodePid service title synopsis availability broadcast parents images'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'BBCRadioRecorder'!

!BBCRadioAODEntry commentStamp: 'jrm 5/7/2017 11:17' prior: 0!
A BBCRadioAODEntry manages the base information that is required to provide information to application pick lists, for choosing a program to record. Instance vars are populated from xmlElements in and AOD availability file from the BBC e.g. "http://www.bbc.co.uk/radio/aod/availability/radio4extra.xml" . Some instance variables are loaded as strings, some as xmlElements for later use (could be designed better, I suppose ...)

Instance Variables
	availability:		<xmlElement>
	broadcast:		<xmlElement>
	images:		<xmlElement>
	key:		<String>
	parents:		<xmlElement>
	pid:		<String>
	service:		<String>
	synopsis:		<String>
	title:		<String>

availability
	- the period of time for which the program is available. Part of BBC Rights Management. Start and End DateTime attributes

broadcast
	- not used currently

images
	- Links to small jpg files. 

key
	- not used currently

parents
	- not used currently

pid
	- Program PID, used to identify a link to a JSON file which contains more episode information used 

service
	- The name of the service source. A BBC Service is a broadcast source, station.

synopsis
	- What this program is about.

title
	- Combination of Brand, Series, and Episode (Program) titles
!

!BBCRadioAODEntry class methodsFor: 'as yet unclassified' stamp: 'jrm 5/9/2017 22:54'!
newFrom: xmlElement
	"comment stating purpose of message"

	^ self new newFrom: xmlElement.! !

!BBCRadioAODEntry methodsFor: 'as yet unclassified' stamp: 'jrm 5/8/2017 07:54'!
availability: xmlElement
	"comment stating purpose of message"

	|  |
	availability := xmlElement.! !

!BBCRadioAODEntry methodsFor: 'as yet unclassified' stamp: 'jrm 5/10/2017 14:40'!
availabilityEnd

	
	^ ((availability attributeAt: #end) first: 10) select: [:each | each isDigit]! !

!BBCRadioAODEntry methodsFor: 'as yet unclassified' stamp: 'jrm 5/10/2017 14:38'!
availabilityStart

	
	^ ((availability attributeAt: #start) first: 10) select: [:each | each isDigit]! !

!BBCRadioAODEntry methodsFor: 'as yet unclassified' stamp: 'jrm 6/27/2017 11:09'!
availabilityStartDateAndTime

	
	^ (availability attributeAt: #start)asDateAndTime! !

!BBCRadioAODEntry methodsFor: 'as yet unclassified' stamp: 'jrm 5/10/2017 16:13'!
canonicalPid

	
	^ canonicalPid! !

!BBCRadioAODEntry methodsFor: 'as yet unclassified' stamp: 'jrm 5/10/2017 16:16'!
episodePid

	
	^ episodePid! !

!BBCRadioAODEntry methodsFor: 'as yet unclassified' stamp: 'jrm 5/10/2017 16:04'!
newFrom: xmlElement
	"comment stating purpose of message"

	|  |
	
	"canonicalPid key pid service title synopsis availability broadcast parents images"

	canonicalPid := (xmlElement at: #pid).
	key := (xmlElement  elementAt: #key) contentString.
	episodePid := (xmlElement  elementAt: #pid) contentString.
	service := (xmlElement  elementAt: #service) contentString.
	title := (xmlElement  elementAt: #title) contentString.
	synopsis := (xmlElement  elementAt: #synopsis) contentString.
	availability :=  (xmlElement  elementAt: #availability).	
	broadcast := (xmlElement  elementAt: #broadcast).
	parents := (xmlElement  elementAt: #parents).
	images :=  (xmlElement  elementAt: #images).	
	
	^self ! !

!BBCRadioAODEntry methodsFor: 'as yet unclassified' stamp: 'jrm 5/8/2017 07:52'!
parents: xmlElement
	"comment stating purpose of message"

	|  |
	parents := xmlElement.! !

!BBCRadioAODEntry methodsFor: 'as yet unclassified' stamp: 'jrm 5/12/2017 17:45'!
scheduleMorph
	"create a single StringMorph for a single schedule entry."

	| tMorph t1 t2 t3 |	
		tMorph := StringButtonMorph new.
	t1 :=  title.
	t2 := self availabilityStart.
	t3 := (synopsis).
	
	^tMorph contents: (t2, ' ', t1); setBalloonText: t3; balloonColor: (Color yellow); balloonFont: (StrikeFont familyName: 'Helvetica' size: 14).! !

!BBCRadioAODEntry methodsFor: 'as yet unclassified' stamp: 'jrm 5/10/2017 16:30'!
synopsis

	
	^ synopsis! !

!BBCRadioAODEntry methodsFor: 'as yet unclassified' stamp: 'jrm 5/10/2017 15:19'!
title

	
	^ title! !

Object subclass: #BBCRadioAvailability
	instanceVariableNames: 'outlet startDtime endDtime episodes'
	classVariableNames: 'OutletUrlStub'
	poolDictionaries: ''
	category: 'BBCRadioRecorder'!

!BBCRadioAvailability commentStamp: 'jrm 5/4/2017 20:17' prior: 0!
This whole class needs to be rewritten due to changes at the BBC.
Start with "http://www.bbc.co.uk/radio/aod/availability/radio4extra.xml" for episode source information. I don't know how long it will be accessible, but it is better than nothing at this point.

A BBCRadioAvailabilitySchedule is the collection of broadcasts which are currently (at the time the object in instantiated) available for listening. The BBC Calls this Listen Again. Most/all programs are supposed to be available for seven(30 now? jrm 20150119) days following the datetime of broadcast. Since a specific program can be re-broadcast within a few days or hours of each other, we must chose the latest broadcast date in the collection to create a single instance for that show. BBC provides weekly schedules by outlet.

Availability is also provided by genra for Iplayer. Schedule availability is limited to schedule entries, but there are programs available in Iplayer that are not on the current schedules

This class will allow availability to be designated by an availabilityMode of #schedule or genra.

Class Variables

OutletUrlStub
	- A Dictionary mapping outlet names to the schedule url stub for the outlet. We do this because there is a url for 'this_week' and 'last_week' with a variety of stubs.

Instance Variables

broadcasts
	- An OrderedCollection of broadcast objects which contain all of the information required to generate a command line string to be executed by the OS. (Sqeak cannot currently record audio streams.

endDtime
	- The datetime, in Z, of the end of the schdule coverage period. Determined by the latest availability date for any broadcast in the collection
	
startDtime
	- The datetime, in Z, of the start of the schdule coverage period. Determined by the earliest availability date for any broadcast in the collection
	
outlet
	- The BBC Radio station identifier for the schedule.

outlet, start and endDtime are derived from the content of broadcasts.
!

!BBCRadioAvailability class methodsFor: 'class initialization' stamp: 'jrm 3/1/2014 13:09'!
initialize
	"BBCRadioAvailabilitySchedule initialize" 
	super initialize.
	OutletUrlStub := (Dictionary new).
	OutletUrlStub
		at: 'radio4extra' put: 'http://www.bbc.co.uk/radio4extra/programmes/schedules/';
		at: 'radio3' put: 'http://www.bbc.co.uk/radio3/programmes/schedules/';
		at: 'radio4' put: 'http://www.bbc.co.uk/radio4/programmes/schedules/fm/';
		at: 'radio2' put: 'http://www.bbc.co.uk/radio2/programmes/schedules/';
		at: '6music' put: 'http://www.bbc.co.uk/6music/programmes/schedules/';
		at: 'radio1' put: 'http://www.bbc.co.uk/radio1/programmes/schedules/england/';
		at: '1xtra' put: 'http://www.bbc.co.uk/1xtra/programmes/schedules/';
		yourself



	
! !

!BBCRadioAvailability class methodsFor: 'instance creation' stamp: 'jrm 3/1/2014 12:58'!
newFor: anOutlet
	"There is no one standard pattern for an outlet, so we build the appropriate urlStub here and create a new instance that will populate itself using the stub."

	"BBCRadioAvailabilitySchedule newFor: 'radio4extra' "

	|  newSchedule|
	newSchedule := self new.
	anOutlet = nil ifFalse:[
	^ newSchedule newFrom:  (OutletUrlStub at: anOutlet)]


	
! !

!BBCRadioAvailability class methodsFor: 'instance creation' stamp: 'jrm 3/1/2014 12:44'!
newFrom: aUrl
	"Build a schedule of all currently available programs. The tag, <is_available_mediaset_pc_sd>0</is_available_mediaset_pc_sd> indicates that the broadcasts is not available >1< means it is. Get data from 'this_week' and 'last_week'"

	"BBCRadioAvailabilitySchedule newFrom: 'http://www.bbc.co.uk/radio4extra/programmes/schedules/this_week.xml' "

	|  newSchedule |

	newSchedule := self new.
	
	^ newSchedule newFrom: aUrl


	
! !

!BBCRadioAvailability class methodsFor: 'instance creation' stamp: 'jrm 1/17/2015 21:34'!
newFromGenra: aCategoryString
	"Build a schedule of all currently available iplayer programs in the given category"

	"BBCRadioAvailabilitySchedule newFromGenra: 'drama/crime'"

	|  newSchedule |

	newSchedule := self new.
	
	^ newSchedule newFromGenra: aCategoryString


	
! !

!BBCRadioAvailability class methodsFor: 'instance creation' stamp: 'jrm 1/31/2015 13:52'!
newFromSchedule: aUrl
	"Build a schedule of all currently available programs. The tag, <is_available_mediaset_pc_sd>0</is_available_mediaset_pc_sd> indicates that the broadcasts is not available >1< means it is. Get data from 'this_week' and 'last_week'"

	"BBCRadioAvailability newFromSchedule: 'http://www.bbc.co.uk/radio4extra/programmes/schedules/this_week.xml' "

	|  newSchedule |

	newSchedule := self new.
	
	^ newSchedule newFromSchedule: aUrl


	
! !

!BBCRadioAvailability methodsFor: 'as yet unclassified' stamp: 'jrm 1/25/2015 14:40'!
compressSchedule
	"Compress the schedule by removing multiple broadcast entries for a single title (shows
	are broadcast more than once)"
	
	| aDictionary temp anOrderedCollection |
	
	aDictionary := Dictionary new.
	temp := episodes collect: [:each | aDictionary at: each title put: each].
	anOrderedCollection := OrderedCollection new.
	aDictionary valuesDo: [ :each | anOrderedCollection add: each].
	episodes := anOrderedCollection.
	! !

!BBCRadioAvailability methodsFor: 'as yet unclassified' stamp: 'jrm 1/25/2015 14:41'!
episodes

	
		^ episodes
	! !

!BBCRadioAvailability methodsFor: 'as yet unclassified' stamp: 'jrm 1/25/2015 14:41'!
episodes: anOrderedCollection

	
		^ episodes := anOrderedCollection
	! !

!BBCRadioAvailability methodsFor: 'as yet unclassified' stamp: 'jrm 5/12/2017 11:09'!
newFrom: aUrlStub
	" "

	|  xmlStream anXMLDocument root |

	episodes := OrderedCollection new.
	"Get 'this_week'"
	{ 'this_week' . 'last_week'} do: [:each | 
	
	xmlStream := HTTPClient httpGet: aUrlStub, each, '.xml'.
	anXMLDocument  := XMLDOMParser parseDocumentFrom: (xmlStream).
	root := anXMLDocument root.
	root tagsNamed: #'day' childrenDo: [:every | every tagsNamed: #'broadcast' do: [:one | 
			
	episodes add: (BBCRadioBroadcastTBD newFrom: one) ]]].
	episodes removeAllSuchThat: [:each | each == nil].
	self compressSchedule.
	episodes sort: [:a :b | a title  > b title].


	
! !

!BBCRadioAvailability methodsFor: 'as yet unclassified' stamp: 'jrm 1/29/2015 07:04'!
newFromGenra: aGenraString
	"aGenraString is used to build the url for an Available Now xml element"

	"http://http://www.bbc.co.uk/radio/programmes/genres/drama/actionandadventure/player/episodes.xml?all"

"BBCRadioAvailability newFromGenra: 'drama/scifiandfantasy'"

	|  xmlStream anXMLDocument root episodesUrl |

	episodes := OrderedCollection new.
	episodesUrl := 'http://www.bbc.co.uk/radio/programmes/genres/', aGenraString, '/player/episodes.xml?all'.
	xmlStream := HTTPClient httpGet: episodesUrl.

	anXMLDocument  := XMLDOMParser parseDocumentFrom: (xmlStream).
	root := anXMLDocument root.
	root tagsNamed: #'day' childrenDo: [:every | every tagsNamed: #'broadcast' do: [:one | 		
	episodes add: (self newFrom: one)]].


	^ episodes sort: [:a :b | a title  > b title].! !

!BBCRadioAvailability methodsFor: 'as yet unclassified' stamp: 'jrm 2/6/2015 17:13'!
newFromPid: episodePid
	"aGenraString is used to build the url for an Available Now xml element"

	"http://http://www.bbc.co.uk/radio/programmes/genres/drama/actionandadventure/player/episodes.xml?all"

"BBCRadioAvailability newFromGenra: 'drama/scifiandfantasy'"

	|  xmlStream anXMLDocument root episodesUrl |

self halt.
"
	episode := .
	episodesUrl := 'http://www.bbc.co.uk/radio/programmes/genres/', aGenraString, '/player/episodes.xml?all'.
	xmlStream := HTTPClient httpGet: episodesUrl.

	anXMLDocument  := XMLDOMParser parseDocumentFrom: (xmlStream).
	root := anXMLDocument root.
	root tagsNamed: #'day' childrenDo: [:every | every tagsNamed: #'broadcast' do: [:one | 		
	episodes add: (self newFrom: one)]].


	^ episodes sort: [:a :b | a title  > b title]."! !

!BBCRadioAvailability methodsFor: 'as yet unclassified' stamp: 'jrm 1/31/2015 13:55'!
newFromSchedule: aUrlStub
	"Build a schedule of all currently available programs. The tag, <is_available_mediaset_pc_sd>0</is_available_mediaset_pc_sd> indicates that the broadcasts is not available >1< means it is. Get data from 'this_week' and 'last_week' using aUrlStub"

	"BBCRadioAvailability newFrom: 'http://www.bbc.co.uk/radio4extra/programmes/schedules/' "

	|  xmlStream anXMLDocument root |

	episodes := OrderedCollection new.
	"Get 'this_week'"
	{ 'this_week' . 'last_week'} do: [:each | 
	
	xmlStream := HTTPClient httpGet: aUrlStub, each, '.xml'.
	anXMLDocument  := XMLDOMParser parseDocumentFrom: (xmlStream).
	root := anXMLDocument root.
	root tagsNamed: #'day' childrenDo: [:every | every tagsNamed: #'broadcast' do: [:one | 
			
	episodes add: (BBCRadioBroadcast newFrom: one) ]]].
	episodes removeAllSuchThat: [:each | each == nil].
	self compressSchedule.
	episodes sort: [:a :b | a title  > b title].


	
! !

Object subclass: #BBCRadioMediaSelector
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'BBCRadioRecorder'!

!BBCRadioMediaSelector class methodsFor: 'accessing' stamp: 'jrm 7/31/2016 18:22'!
buildMediaSelectorString: anEpisodePid fileName: fileName

" build and execute command line for RTMPDUMP; ffmpeg, and rm" 

	| cmdLine mediaAac mediaDocument dQuote episodeUrl episodeDocument  connection  supplier application authExpires authString identifier priority protocol server flvFileName mp4Filename canonicalPid  episodeStream mediaStream defaultDestination randomString eosp |
	defaultDestination := BBCRecorderApp defaultDestination.
	randomString := (((Random new next ) * 100000000) asInteger) asString.
	dQuote := '"'.

	episodeUrl := 'http://www.bbc.co.uk/programmes/', anEpisodePid, '.xml'.


	episodeStream :=HTTPClient httpGet: episodeUrl.	
	episodeDocument  := XMLDOMParser parseDocumentFrom: (episodeStream).
	canonicalPid := ((episodeDocument firstTagNamed: #versions) firstTagNamed: #pid )contentString.
	"mediaStream:= HTTPClient httpGet: 'http://www.bbc.co.uk/mediaselector/4/mtis/stream/', canonicalPid.  >>> depreciated url prefix"
	mediaStream:= HTTPClient httpGet: 'http://open.live.bbc.co.uk/mediaselector/5/select/version/2.0/mediaset/pc/vpid/', canonicalPid.
	mediaDocument := XMLDOMParser parseDocumentFrom: (mediaStream).
	mediaAac := mediaDocument firstTagNamed: #media with: [:each | (each attributeAt: #bitrate) = '48'].
 	connection := (mediaAac firstTagNamed: #connection) attributes.

supplier  := connection at: #supplier.
protocol  := connection at: #protocol.
authExpires   := connection at: #authExpires.
authString  := connection at: #authString.
priority  := connection at: #priority.
server  := connection at: #server.
application  := connection at: #application.
identifier  := connection at: #identifier.
flvFileName := randomString, '.flv'.
mp4Filename := fileName.
cmdLine := '/opt/local/bin/rtmpdump -r ' , dQuote, protocol, '://', server, ':1935/',  application, '?', authString, dQuote, ' -a ', dQuote, application, '?', authString, dQuote, ' -y "', identifier, dQuote,  ' -o ', defaultDestination, flvFileName, ' ; /opt/local/bin/ffmpeg -loglevel quiet  -i ', defaultDestination, flvFileName, ' -c copy -copyts ', defaultDestination, mp4Filename, ' ; rm ', defaultDestination, flvFileName.
"Create a shell command file, execute it, and delete it"

eosp := ExternalUnixOSProcess forkAndExec: ExternalUnixOSProcess bashShellPath
		arguments: (Array with: '-c' with: cmdLine)
		environment: nil.

^ eosp! !

Object subclass: #BBCRecorderApp
	instanceVariableNames: 'availabilitySchedule aSystemWindow aScrollPane aPasteUpMorph osp2'
	classVariableNames: 'DefaultOutlet DestinationFolder'
	poolDictionaries: ''
	category: 'BBCRadioRecorder'!

!BBCRecorderApp commentStamp: 'jrm 4/16/2016 10:28' prior: 0!
A BBCRecorderApp is An application to browse and record the Listen Again availability list for various BBC radio stations from outside the UK using the ffmpeg application.

Instance Variables
	aBBCRadioSchedule:		<Object>

availabilitySchedule
	- xxxxx
aBBCRadioWeeklySchedule
	- to be deprecated at some point
SystemWindow 
	- xxxxx
aPasteUpMorph 
	- xxxxx
osp2
	- is this needed as an instance var?


Class Variables

	DestinationFolder          Folder where recordings are written. User Selectable

Check for classes not longer required. There has bee a lot of hacking going on
BBCRadioAvailabilitySchedule allInstances >>> gone now? 
BBCRadioProgramme allInstances >>> gone now? 
BBCRadioBroadcast allInstances
BBCRadioMediaSelector allInstances
BBCRadioSchedule allInstances
BBCRecorderApp allInstances!

!BBCRecorderApp class methodsFor: 'initialize-release' stamp: 'jrm 7/30/2016 14:30'!
defaultDestination

"This will be used for insertion into a command string"
	^ DestinationFolder := '/Volumes/MediaVault/BBC/'
	
"DefaultDestination DefaultFfmpegPath DefaultRtmpdumPath"! !

!BBCRecorderApp class methodsFor: 'initialize-release' stamp: 'jrm 2/7/2014 19:42'!
defaultOutlet

   	^ DefaultOutlet := 'radio4extra'! !

!BBCRecorderApp class methodsFor: 'initialize-release' stamp: 'jrm 8/31/2014 20:11'!
initialize

    "BBCRecorderApp initialize"

 	self defaultOutlet; defaultDestination; registerInWorldMenu.! !

!BBCRecorderApp class methodsFor: 'instance creation' stamp: 'jrm 3/1/2014 12:48'!
openAsMorph
"BBCRecorderApp openAsMorph "

	^self new openAsMorph! !

!BBCRecorderApp class methodsFor: 'initialize-release' stamp: 'jrm 3/1/2014 12:50'!
registerInWorldMenu
	self environment at: #TheWorldMenu ifPresent: [ :class |
		class registerOpenCommand: (Array 
			with: 'BBC Recorder' 
			with: (Array
				with: self
				with: #openAsMorph)) ].! !

!BBCRecorderApp class methodsFor: 'instance creation' stamp: 'jrm 1/17/2015 21:46'!
zOpenAsMorph
"BBCRecorderApp zOpenAsMorph "

	^self new zOpenAsMorph! !

!BBCRecorderApp methodsFor: 'user interface' stamp: 'jrm 8/13/2016 17:23'!
buttonSelected: aStringButtonMorph entrySelected: aBBCRadioBroadcast


BBCEpisodeRecorder recordEpisode: (BBCEpisode newFor: aBBCRadioBroadcast episodePid)! !

!BBCRecorderApp methodsFor: 'user interface' stamp: 'jrm 2/7/2014 19:42'!
composeFileNameFor: aString seriesCount: sCountString episode: episodeString
	"We need a fileName to use in the command line when we record a stream. The file name is based on the title of the stream and we offer the option to edit the title directly or select a file name from a directory and edit it. The second option is most useful for a series when all you want to do is update the episode or series numbers"
	| option editString fullString |
	

	"Clean up problem characters"
	editString := aString replaceAll: $' with: $-.
	editString := editString replaceAll: $:  with: $-.
	editString  := editString replaceAll: $/  with: $-.
	fullString := editString,'.', episodeString,'.', sCountString.
	"Ask for the desired option"
	option := UserDialogBoxMorph confirm: 'File Name for Stream' withCRs title: 'Create new, or look up existing?' trueChoice: 'Lookup' falseChoice: 'Create'.  
	option ifTrue: [ ^ self fileNameFromString: (self fileNameFromDirectory: fullString )] 
			ifFalse:[^ self fileNameFromString: fullString]! !

!BBCRecorderApp methodsFor: 'user interface' stamp: 'jrm 5/12/2017 18:05'!
compressSchedule
	"Compress the schedule by removing multiple entries for a single title (shows
	are broadcast more than once"
	
	| aDictionary temp anOrderedCollection |
	
	aDictionary := Dictionary new.
	temp := availabilitySchedule  collect: [:each | aDictionary at: each title put: each].
	anOrderedCollection := OrderedCollection new.
	aDictionary valuesDo: [ :each | anOrderedCollection add: each].
	^ availabilitySchedule.
	! !

!BBCRecorderApp methodsFor: 'user interface' stamp: 'jrm 3/1/2014 12:48'!
currentSchedule
	""
	
	^ availabilitySchedule! !

!BBCRecorderApp methodsFor: 'user interface' stamp: 'jrm 2/12/2014 20:54'!
defaultOutlet
	
	
	^ 'radio4extra'! !

!BBCRecorderApp methodsFor: 'user interface' stamp: 'jrm 12/9/2014 07:18'!
fileNameFromDirectory
	" Let me select a name from a list so I can keep the file names for a list similar. I will edit the file 	name that I select in a later step"
	"BBCRecorderApp fileNameFromList"
	
	| fileList fileNameLIst fileName|
	fileList := (FileList newOn: (FileDirectory on: '/Volumes/MediaVault/BBC')) entriesMatching: '*.mp4'.
	fileNameLIst := fileList collect: [ :each | each name].
	fileName := ListChooser chooseItemFrom: fileNameLIst.
	"Strip file type indicator string .mp3 from end of file"
	^ fileName == nil ifTrue: [^fileName] ifFalse: [^ fileName copyFrom: 1 to: (fileName size -4)]
! !

!BBCRecorderApp methodsFor: 'user interface' stamp: 'jrm 2/7/2014 19:42'!
fileNameFromDirectory: aString
	" Let me select a name from a list so I can keep the file names for a list similar. I will edit the file 	name that I select in a later step"
	"BBCRecorderApp fileNameFromList"
	
	| fileList fileNameLIst fileName|
	fileList := (FileList newOn: (FileDirectory on: '/Volumes/MediaVault/BBC')) entriesMatching: '*.mp3'.
	fileNameLIst := fileList collect: [ :each | each name].
	fileName := ListChooser chooseItemFrom: fileNameLIst title: 'Hint: ', aString.
	"Strip file type indicator string .mp3 from end of file"
	^ fileName == nil ifTrue: [^fileName] ifFalse: [^ fileName copyFrom: 1 to: (fileName size -4)]
! !

!BBCRecorderApp methodsFor: 'experiment' stamp: 'jrm 2/7/2014 19:42'!
fileNameFromString: aString seriesCount: sCountString episode: episodeString
	"Modify the title string to create a useful file name to store the transcribed stream."
	| aNewString fileName |
" (BBCRecorderApp new) fileNameFromString: 'Some thing new: series 5 3.5'"

	"Dump spaces from aString"
	aNewString := aString  select: [ :each | each value ~= Character space].
	episodeString = nil ifTrue:[ fileName := aNewString ] ifFalse: [fileName := aNewString,'.', episodeString,'.', sCountString].
      fileName :=	FillInTheBlankMorph  request:  '.raw will be appended to file name' initialAnswer: fileName centerAt: 500@500.	
	fileName := '"',fileName,  '.raw','"'.

self halt.	
	! !

!BBCRecorderApp methodsFor: 'initalization' stamp: 'jrm 2/7/2014 19:42'!
initialize
	"comment stating purpose of message"
	! !

!BBCRecorderApp methodsFor: 'user interface' stamp: 'jrm 2/7/2014 19:42'!
menuFiles
	"Display the file list for review. User can copy string if desired???"
	
	
	
self fileNameFromDirectory.

! !

!BBCRecorderApp methodsFor: 'user interface' stamp: 'jrm 5/12/2017 17:59'!
menuOutlet
	"The BBC Radio Stations are called outlets. All we do here is get the new outlet and use it to set the aBBCRadioSchedule"
	

	
	
	| outlet |
	outlet := MenuMorph
		chooseFrom: #('radio4extra' 'radio4'  'radio3' 'radio2' '6music'  'radio1' '1xtra')
		values: #('radio4extra' 'radio4' 'radio3' 'radio2' '6music' 'radio1' '1xtra')
		lines: #()
		title: 'Outlet'.
	outlet = nil ifFalse: [
	 availabilitySchedule := (BBCRadioAOD newFor: outlet).
	 aPasteUpMorph removeAllMorphs.
	self populateSchedule; compressSchedule.]
! !

!BBCRecorderApp methodsFor: 'user interface' stamp: 'jrm 2/13/2016 15:39'!
menuSort
	"Display a menu of sort options Alpha Date. Perform the sort"
	
	
	| sortMethod |
	sortMethod := MenuMorph
		chooseFrom: #('Alpha' 'Date')
		values: #('sortAlpha' 'sortAvailabilityStartDate')
		lines: #()
		title: 'Sort by'.
	sortMethod = nil ifFalse: [ self perform: sortMethod asSymbol]
	
	

! !

!BBCRecorderApp methodsFor: 'user interface' stamp: 'jrm 5/12/2017 17:34'!
openAsMorph
	
	"BBCRecorderApp openAsMorph "
	| submorphExtent menuButton1 menuButton2 mbHolder menuButton3 |
	
	availabilitySchedule := BBCRadioAOD newFrom: 'http://www.bbc.co.uk/radio/aod/availability/radio4extra.xml'  .
	aSystemWindow := SystemWindow labelled: 'BBC'.
	aSystemWindow model: self.

mbHolder := (RectangleMorph new) layoutPolicy: TableLayout new;
	  listDirection: #leftToRight;
	 wrapCentering: #topLeft;
	  hResizing: #shrinkWrap;
	  vResizing: #shrinkWrap.
menuButton1 := SimpleButtonMorph new
		target: self ;
		label: 'Sort';
		actionSelector: #menuSort.
menuButton2 := (SimpleButtonMorph new
		target: self;
		label: 'Outlet';
		actionSelector: #menuOutlet;
		setBalloonText: 'Select a BBC Radio Outlet to display a list of available programs').
menuButton3 := (SimpleButtonMorph new
		target: self;
		label: 'FileNameReview';
		actionSelector: #menuFiles;
		setBalloonText: 'Take a look at the file names in the  current destination folder').
		
mbHolder addMorph: menuButton3; addMorph: menuButton1;addMorph: menuButton2.
aSystemWindow addMorph: mbHolder frame:  (0.0@0 corner: 1@0.007).
aPasteUpMorph := Morph new.
aPasteUpMorph color: Color veryVeryLightGray.

aPasteUpMorph layoutPolicy: TableLayout new;
	  listDirection: #topToBottom;
	 wrapCentering: #topLeft;
	  hResizing: #shrinkWrap;
	  vResizing: #shrinkWrap.


aScrollPane := ScrollPane new.
aScrollPane scroller addMorph: aPasteUpMorph.
aPasteUpMorph on: #keyStroke send: #keyStroke: to: aScrollPane. 
aSystemWindow addMorph: aScrollPane frame: (0@0.07 extent: 1@0.93).
submorphExtent := aPasteUpMorph submorphBounds corner.
aScrollPane extent: submorphExtent.
aPasteUpMorph extent: submorphExtent; cellPositioning: #topLeft.
self populateSchedule.
aSystemWindow openInWorld.! !

!BBCRecorderApp methodsFor: 'user interface' stamp: 'jrm 2/22/2014 22:27'!
outletUrl: outlet
	"build the url using the value provided. If the value does not resolve correctl, there will be no file returned."

	outlet= 'radio4' ifTrue: [^ 'http://www.bbc.co.uk/radio4/programmes/schedules/fm/this_week.xml'].
	^ 'http://www.bbc.co.uk/', outlet, '/programmes/schedules/last_week.xml'! !

!BBCRecorderApp methodsFor: 'user interface' stamp: 'jrm 2/23/2014 10:54'!
outletUrlStub: outlet
	"Some of the BBC outlets have slightly different URLs. The final part of the complete string is this_week or last_week. Both URLs have to be accessed to get a complete list of programs available for recording."

	outlet= 'radio4' ifTrue: [^ 'http://www.bbc.co.uk/radio4/programmes/schedules/fm/'].

	^ 'http://www.bbc.co.uk/', outlet, '/programmes/schedules/'! !

!BBCRecorderApp methodsFor: 'user interface' stamp: 'jrm 5/12/2017 18:04'!
populateSchedule
	"Build aPasteUpMorph and add individual program schedule morphs into it.. Run once when a new schedule is selected into currentSchedule  "
	
| aScheduleMorph |
	availabilitySchedule do: [:each   | aScheduleMorph := each scheduleMorph. 
	aScheduleMorph on: #keyStroke send: #keyStroke: to: aScrollPane.
	aPasteUpMorph addMorph: (aScheduleMorph cellPositioning: #topLeft; target: self; actionSelector: #buttonSelected:entrySelected: ; arguments:  (Array with: aScheduleMorph with: each))].
! !

!BBCRecorderApp methodsFor: 'user interface' stamp: 'jrm 5/12/2017 18:08'!
sortAlpha
	"Sort the pick list in alpha order"
	
	aPasteUpMorph submorphsDo: [:each | each delete].
	availabilitySchedule := (availabilitySchedule  sort: [:a :b | (a title, a availabilityEnd) > (b title, b availabilityEnd)]).

	self populateSchedule.! !

!BBCRecorderApp methodsFor: 'user interface' stamp: 'jrm 5/12/2017 18:14'!
sortAvailabilityStartDate
	"Sort the pick list in availability startdate order"
	
	aPasteUpMorph submorphsDo: [:each | each delete].
	availabilitySchedule:=  (availabilitySchedule  sort: [:a :b | (a availabilityStart) > (b availabilityStart)]).

	self populateSchedule.
	
! !

!BBCRecorderApp methodsFor: 'experiment' stamp: 'jrm 7/30/2016 14:33'!
zOpenAsMorph
	" experiment to test using category rather than scheduled"
	
	"BBCRecorderApp zOenAsMorph "
	| submorphExtent menuButton1 menuButton2 mbHolder menuButton3 |
	
	availabilitySchedule := BBCRadioAvailability newFromGenra: 'drama/crime'.
	aSystemWindow := SystemWindow labelled: 'BBC-Genra'.
	aSystemWindow model: self.

mbHolder := (RectangleMorph new) layoutPolicy: TableLayout new;
	  listDirection: #leftToRight;
	 wrapCentering: #topLeft;
	  hResizing: #shrinkWrap;
	  vResizing: #shrinkWrap.
menuButton1 := SimpleButtonMorph new
		target: self ;
		label: 'Sort';
		actionSelector: #menuSort.
menuButton2 := (SimpleButtonMorph new
		target: self;
		label: 'Outlet';
		actionSelector: #menuOutlet;
		setBalloonText: 'Select a BBC Radio Outlet to display a list of available programs').
menuButton3 := (SimpleButtonMorph new
		target: self;
		label: 'FileNameReview';
		actionSelector: #menuFiles;
		setBalloonText: 'Take a look at the file names in the  current destination folder').
		
mbHolder addMorph: menuButton3; addMorph: menuButton1;addMorph: menuButton2.
aSystemWindow addMorph: mbHolder frame:  (0.0@0 corner: 1@0.007).
aPasteUpMorph := Morph new.
aPasteUpMorph color: Color veryVeryLightGray.

aPasteUpMorph layoutPolicy: TableLayout new;
	  listDirection: #topToBottom;
	 wrapCentering: #topLeft;
	  hResizing: #shrinkWrap;
	  vResizing: #shrinkWrap.


aScrollPane := ScrollPane new.
aScrollPane scroller addMorph: aPasteUpMorph.
aPasteUpMorph on: #keyStroke send: #keyStroke: to: aScrollPane. 
aSystemWindow addMorph: aScrollPane frame: (0@0.07 extent: 1@0.93).
submorphExtent := aPasteUpMorph submorphBounds corner.
aScrollPane extent: submorphExtent.
aPasteUpMorph extent: submorphExtent; cellPositioning: #topLeft.
self populateSchedule.
aSystemWindow openInWorld.! !
BBCRadioAvailability initialize!
BBCRecorderApp initialize!
